SystemOrganization addCategory: #'Sea28WA-Core'!

Announcer subclass: #WAAnnouncer
	instanceVariableNames: 'contents parentComponent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sea28WA-Core'!

!WAAnnouncer commentStamp: '<historical>' prior: 0!
"Wandering Announcements" inSeaside.

(This design is based upon Lukas' simple Announcements framework)

Whereas Announcements are often used with a centralized Announcer, Wandering announcements may
look for someone to respond to their message in the given surroundings. They may "bubble up" a component hierarchy, or interrogate siblings, or decorations as they wish.

Each WAComponent in seaside is a node in this communications network. Nodes function as an announcer, and as a source of potential routes for wandering announcements to follow.

Routing is provided by #ann* methods, which are private to the Wandering Announcer, no one else should call them.
e.g. #annParent #annSiblingsIn: [ :siblings | .... ].  #annChildrenIn: [ :children | .... ].

Unlike normal announcers which figure out what to announce to whom, as much responsibility is given to each Announcement as possible so that behaviour can be overriden completely for different scenarios.

When handling an announcement we double back and send #announceTo: to the announcement itself to give it complete control, over what it does before or after informing dependants. It passes straight back to the anouncers #dependantsAnnounce: for the default behavior. This allows an Announcement to replace the behaviour of Announcer-#dependantsAnnounce: altogether if it wants to.

Triggering example: (we always return the original announcer)

theAnnouncement := self announcer announce: (SomeAnnouncement param: arg1).

Subscriptions example:

self announcer on: SomeAnouncement do: [ :announcement | ...  ].
self announcer on: SomeAnouncement send: #tellMe: to: anObject.

Normally, the announcer is passed as the argument, however, each announcement supplies the #args, so it can call a method that is not expecting an Announcement as the parameter. This enables you to wire up objects that were not designed with announcements in mind.

example trigger:
self announce: (HtmlUpdateAnnouncement on: html)

self announcer on: HtmlUpdateAnouncement send: #renderOn: to: anObject.

where:

HtmlUpdateAnnouncement-#args
^ Array with: html

====
Notes on this class
=====
Components dont have anywhere to keep properties, since every component has a WAValueHolder we steal his action and may as well keep additional properties in here - and we may as well be an announcer too, after all its only behavior, if we arent used our state will be nil.

The inst var decoration in WAComponent should be renamed to something else so

WAComponent decoration
	^ decoration contents
	
becomes
	
WAComponent-decoration
	^ extn decoration   
	
Siblings need to be cached while we are hunting through them, #children doesnt build any relationships at all, so we must use #childrenDo: ...

So process siblings like so, and announce them to others who may be interested

self annSiblingsIn: [ :siblings |

].!

!WAAnnouncer class methodsFor: 'as yet unclassified' stamp: 'kph 9/3/2008 01:00'!
with: anObject
	^ self new contents: anObject! !

!WAAnnouncer methodsFor: 'search directions' stamp: 'kph 4/25/2008 08:32'!
annChildrenIn: aBlock 
	"if an announcer is looking at children it needs to use
	#childrenDo: rather than #children because children doesnt build any relationships"

	| children |
	children := OrderedCollection new.
	
	self component childrenDo: [ :ea |
		children add: ea.
	].

	aBlock value: children.
	
	"
	usage: 
	
	I want to look through my siblings, or announce stuff to my siblings.
	this way the list of siblings is cached whiile the query is occuring.
		
	self annParent annChildrenIn: [ :siblings |
		
		
	].

	"! !

!WAAnnouncer methodsFor: 'search directions' stamp: 'kph 4/25/2008 08:32'!
annNextSiblingInList: siblings

	^ siblings after: self ifAbsent: nil! !

!WAAnnouncer methodsFor: 'search directions' stamp: 'kph 4/25/2008 09:26'!
annParent 
	^ self component annParent  ! !

!WAAnnouncer methodsFor: 'search directions' stamp: 'kph 4/25/2008 08:33'!
annPrevSiblingInList: siblings

	^ siblings before: self ifAbsent: nil! !

!WAAnnouncer methodsFor: 'search directions' stamp: 'kph 7/18/2008 01:31'!
annSiblingsIn: aBlock 
	^ self component annParent annChildrenIn: aBlock! !

!WAAnnouncer methodsFor: 'announce' stamp: 'kph 4/25/2008 05:36'!
announce: anAnnouncement
 
	^ (anAnnouncement asAnnouncement)
		announcedTo: self;
		yourself
	
	 ! !

!WAAnnouncer methodsFor: 'announce' stamp: 'kph 7/18/2008 01:42'!
component

	^ contents isDecoration
		ifTrue: [ contents component]
		ifFalse: [ contents ]! !

!WAAnnouncer methodsFor: 'accessing' stamp: 'kph 7/18/2008 01:40'!
contents
	
	^ contents! !

!WAAnnouncer methodsFor: 'accessing' stamp: 'kph 7/18/2008 01:41'!
contents: anObject
	
	contents := anObject! !

!WAAnnouncer methodsFor: 'framework' stamp: 'kph 4/25/2008 07:24'!
parentComponent
	^ parentComponent! !

!WAAnnouncer methodsFor: 'framework' stamp: 'kph 4/25/2008 07:25'!
setParentComponent: aComponent

	parentComponent := aComponent! !

!WAAnnouncer methodsFor: 'announce' stamp: 'kph 4/25/2008 07:34'!
subscriptions
	^ subscriptions! !

!WAComponent methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 09:32'!
annChildrenIn: aBlock 
	"here we publish our 'conceptual' children for sideways wanderings, 
	this doesnt have to contain all of the actual children"

	| children |

	children := OrderedCollection new.
	
	self childrenDo: [ :ea |
		children add: ea.
	].

	aBlock value: children.
	
	"
	usage: 
	
	I want to look through my siblings, or announce stuff to my siblings.
	this way the list of siblings is cached whiile the query is occuring.
		
	self annParent annChildrenIn: [ :siblings |
		
		
	].

	"! !

!WAComponent methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 07:29'!
annParent
	"our announcer knows our actual parentComponent.
	we publich our conceptual parent"
	^ self announcer parentComponent! !

!WAComponent methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 03:59'!
announcer

	^ decoration ! !

!WAComponent methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 07:28'!
childrenDo: aBlock

 	self children do: [:ea | ea ifNotNil: [ 
		
		ea setActualParentComponent: self. 		
		aBlock value: ea

	]]! !

!WAComponent methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 03:08'!
initializeDecoration
	decoration := WAAnnouncer with: self! !

!WAComponent methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 07:29'!
setActualParentComponent: parent
	
	decoration setParentComponent: parent! !

!Announcement methodsFor: '*sea28wa-core' stamp: 'mtf 7/17/2008 18:58'!
announcedTo: announcer
"Dispatches the announcement in whatever manner is appropriate. Perhaps this method should be called ""announceVia: announcer"", but it isn't. Override me to change the dispatch policy. By default, the announcement is dispatched to all subscribers to the announcer, disregarding any semantic arrangement"

	self dependantsAnnounceTo: announcer! !

!Announcement methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 06:31'!
args
	^ Array with: self
	! !

!Announcement methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 08:13'!
dependantsAnnouncedTo: announcer

	announcer subscriptions ifNotNil: [ 
		announcer subscriptions keysAndValuesDo: [ :class :actions |
			(class handles: self) 
				ifTrue: [  
						(actions detectWithArguments: self args) ifTrue: [ ^ true ] ] ].
	].

	^ false
 	 ! !

Announcement subclass: #WABubbleUp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sea28WA-Core'!

!WABubbleUp commentStamp: 'kph 4/25/2008 09:19' prior: 0!
Handling of announcements is terminated whenever a handling block/method returns true.!

!WABubbleUp class methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 05:29'!
selector: aSelector

	^ self new setMessage: (Message selector: aSelector)! !

!WABubbleUp methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 07:47'!
announcedTo: announcer
 	 
	self informDependants ifTrue: [ (self dependantsAnnounceTo: announcer) ifTrue: [ ^ self ] ].
		 	
	announcer annParent ifNotNilDo: [ :parent | parent announcer announce: self  ].
	! !

!WABubbleUp methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 09:10'!
informDependants
 	 
	^ true! !

WABubbleUp subclass: #WALookupUpwards
	instanceVariableNames: 'selector result block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sea28WA-Core'!

!WALookupUpwards commentStamp: 'kph 4/25/2008 08:28' prior: 0!
"WA" Wandering Announcer lookup query, which bubbles up the tree:

Use case... I want to enquire of my parents if there is a form Id... so, I might not be interested in any subscriptions

I ask my parents if they respond to a selector, if so the found target is passed to the detect block.
When the detect block returns true, the process stops.

Examples:

perform #a:b: on all subscribers.
 
(aComponent announce:  (WALookUp selector: #a:b:)) detect: [ :found | found a: 1 b: 2). false ].

Items what are not in the direct search path can register themselves as able to respond like so:

self announcer on: WALookupUp do: [ :ann |  ann query: me   ]

The item which was handling the block which returned true, is kept and is available via #result. This could be useful if you are looking for an object with a flag set eg:

foundItem := ((aComponent announce:  (WALookUp selector: #isSelected)) detect: [ :found | found isSelected ]) result.


!

WALookupUpwards subclass: #WALookupSideways
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sea28WA-Core'!

!WALookupSideways methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 08:30'!
announcedTo: announcer

	"when we arent interested in dependants we return false."
	
	"we ask the parent if it has an answer, if not we pass the announcement on"
	
	(self query: announcer annParent) ifTrue: [ ^ self ].
	 
	self informDependants ifTrue: [ (self dependantsAnnounceTo: announcer) ifTrue: [ ^ self ] ].
		 	
	 announcer annParent ifNotNilDo: [ :parent | parent announce: self  ].
	! !

!WALookupUpwards class methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 07:21'!
selector: aSelector

	^ self basicNew 
		setSelector: aSelector;
		initialize;
		yourself! !

!WALookupUpwards methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 07:49'!
announcedTo: announcer

	"when we arent interested in dependants we return false."
	
	"we ask the parent if it has an answer, if not we pass the announcement on"
	
	(self query: announcer annParent) ifTrue: [ ^ self ].
	 
	self informDependants ifTrue: [ (self dependantsAnnouncedTo: announcer annParent announcer) ifTrue: [ ^ self ] ].
		 	
	announcer annParent ifNotNilDo: [ :parent | parent announcer announce: self  ].
	! !

!WALookupUpwards methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 07:46'!
detect: aBlock

	block := aBlock! !

!WALookupUpwards methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 08:24'!
query: target
 
	^ (target respondsTo: selector)
		ifTrue: [ block value: (result := target) ] ifFalse: [ false ].
	 ! !

!WALookupUpwards methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 08:24'!
result

^ result! !

!WALookupUpwards methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 08:15'!
setSelector: aSymbol

	selector := aSymbol! !

!ActionSequence methodsFor: '*sea28wa-core' stamp: 'kph 4/25/2008 08:13'!
detectWithArguments: anArray  

	"apply until we get a 'true' result"
	
    self do:
        [:each |
         (each valueWithArguments: anArray) ifTrue: [ ^ true ] ].

	^ false! !
